use std::io;

use async_compression::tokio::bufread::GzipDecoder;
use rocket::{
    data::{Data, ToByteUnit},
    delete, post, routes, Route, State,
};
use tokio::io::{AsyncBufRead, AsyncRead, BufReader};
use tokio_stream::StreamExt;
use tokio_tar::Archive;
use tokio_util::io::ReaderStream;

use crate::Config;
use crate::Websites;

pub fn routes() -> Vec<Route> {
    routes![subdomain_delete, subdomain_upload]
}

#[delete("/subdomain/<subdomain>")]
pub fn subdomain_delete(subdomain: String, hosts: &State<Websites>) {
    println!("delete to subdomain {}", subdomain);
}

#[post(
    "/subdomain/<subdomain>/upload",
    format = "application/x-gtar",
    data = "<data>"
)]
pub async fn subdomain_upload(
    subdomain: String,
    data: Data,
    hosts: &State<Websites>,
    config: &State<Config>,
) {
    // TODO: remove all unwraps. What to return ? option is easy, error can be logged
    // TODO: move directory creation the Folder backend manager
    let domain_path = match hosts.subdomain_path(&subdomain).unwrap() {
        None => hosts.add_subdomain(&subdomain).unwrap(),
        Some(path) => path,
    };
    let path = config.data_dir.join(&domain_path);
    println!(
        "upload to subdomain {}, which is stored in path {:?}",
        subdomain, path
    );
    let data = data.open(5.megabytes());
    // FIXME: convert using BufReader may be slow, is it possible to obtain a more efficient
    // AsyncBufRead from rocket::Data ?
    let tar = GzipDecoder::new(BufReader::new(data));
    let mut archive_reader = Archive::new(tar);

    tokio::fs::create_dir_all(&path).await.unwrap();
    archive_reader.unpack(&path).await.unwrap();
}
